@@ -43,10 +43,6 @@ TipQalc::TipQalc(QWidget *parent) :
     connect(ui->actionAbout, SIGNAL(triggered()),
             this, SLOT(showAboutDialog()));
 
-    QCoreApplication::setOrganizationName("SeviSoft");
-    QCoreApplication::setOrganizationDomain("sevisoft.com");
-    QCoreApplication::setApplicationName("TipQalc");
-
     QSettings settings;
     //settings.clear();
 
@@ -63,13 +59,11 @@ TipQalc::TipQalc(QWidget *parent) :
     }
 
     manageTaxInputWidgets();
-    // remove as soon as functionality is complete
-    ui->roundButton->hide();
+    resetResults();
 }
 
 TipQalc::~TipQalc()
 {
-    QSettings settings;
     settings.setValue("taxPercent", ui->taxLine->text());
     settings.setValue("tipPercent", ui->tipLine->text());
     delete ui;
@@ -97,8 +91,8 @@ void TipQalc::showSettingsDialog()
     SettingsDialog settingsDialog(this);
     settingsDialog.exec();
 
-    if (total) calculateTip();
     manageTaxInputWidgets();
+    if (total) calculateTip();
 }
 
 /*
@@ -115,18 +109,14 @@ void TipQalc::showAboutDialog()
 */
 bool TipQalc::eventFilter(QObject *object, QEvent *event)
 {
-    if (event->type() == QEvent::FocusIn)
-    {
+    if (event->type() == QEvent::FocusIn) {
         editableLine = qFindChild<QLineEdit*>(this, object->objectName());
         return true;
     }
-    else
-    {
+    else {
         // standard event processing
         return QObject::eventFilter(object, event);
     }
-    
-
 }
 
 
@@ -135,16 +125,13 @@ bool TipQalc::eventFilter(QObject *object, QEvent *event)
 */
 void TipQalc::manageTaxInputWidgets()
 {
-    QSettings settings;
-    bool useTax = settings.value("subtractTaxFromTotal").toBool();
-    if (!useTax)
-    {
+    useTax = settings.value("subtractTaxFromTotal").toBool();
+    if (!useTax) {
         ui->taxLabel->hide();
         ui->taxLine->hide();
         editableLine = ui->billLine;
     }
-    else
-    {
+    else {
         ui->taxLabel->show();
         ui->taxLine->show();
     }
@@ -189,8 +176,64 @@ void TipQalc::adjustTip(int value)
 */
 float TipQalc::round( float x, int precision=2)
 {
-    float sd = pow(10, precision);
-    return int(x*sd + (x<0? -0.5 : 0.5))/sd;
+    float p = pow(10, precision);
+    return int(x*p + (x<0 ? -0.5 : 0.5)) / p;
+}
+
+/*
+  Weighted rounding method.
+*/
+float TipQalc::roundWeighted( float x)
+{
+    return int(x + (x<0 ? -0.35 : 0.65));
+}
+
+/*
+  Rounding to nearest quarter.
+*/
+float TipQalc::roundToQuarter(float x)
+{
+    x = round(x, 2);
+    float y = x - (int)x;
+
+    if (y <= 0.12){
+        y = 0;
+    }
+    else if (y > 0.12 && y <= 0.37){
+        y = 0.25;
+    }
+    else if (y > 0.37 && y <= 0.62){
+        y = 0.5;
+    }
+    else if (y > 0.62 && y <= 0.87){
+        y = 0.75;
+    }
+    else if (y > 0.87) {
+        y = 1;
+    }
+    return (int)x + y;
+}
+
+/*
+  Prepare a float for display.
+ */
+QString TipQalc::floatToDisplay(float n)
+{
+    QString s = QString::number(n);
+    return s.sprintf("%.2f", n);
+}
+
+/*
+  Reset the results display to an initial state.
+*/
+void TipQalc::resetResults()
+{
+    ui->totalTipResultLabel->setText("0");
+    ui->totalResultLabel->setText("0");
+    ui->personTotalResultLabel->setText("0");
+    ui->personTipResultLabel->setText("0");
+    ui->newTipLabel->hide();
+    ui->newTipDisplay->hide();
 }
 
 /*
@@ -198,45 +241,73 @@ float TipQalc::round( float x, int precision=2)
 */
 void TipQalc::calculateTip()
 {
+    float bill = ui->billLine->text().toFloat();
+    if (bill <= 0){
+        resetResults();
+        return;
+    }
+
     int persons = ui->personsLine->text().toInt();
     float taxPercent = ui->taxLine->text().toFloat();
     float tipPercent = ui->tipLine->text().toFloat();
-    float bill = ui->billLine->text().toFloat();
-
-    QSettings settings;
-    bool useTax = settings.value("subtractTaxFromTotal").toBool();
-
     float tip;
-    if (useTax)
-    {
-        float tax = (bill * taxPercent) / 100;
-        tip = ( (bill - tax) * tipPercent ) / 100;
+
+    bool doRoundToQuarter = settings.value("RoundToQuarter").toBool();
+    int totalRounding = ui->roundTotalButton->isChecked();
+    int tipRounding = ui->roundTipButton->isChecked();
+
+    float tax = (bill * taxPercent) / 100;
+    float adjustedBill = useTax ? (bill - tax) : bill;
+
+    tip = ( adjustedBill * tipPercent ) / 100;
+
+
+    // round tip
+    if (tipRounding && doRoundToQuarter){
+        tip = roundToQuarter(tip / persons) * persons;
     }
-    else
-    {
-        tip = ( bill * tipPercent ) / 100;
+    else if (tipRounding){
+        tip = roundWeighted(tip / persons) * persons;
+    }
+    else {
+        tip = round(tip / persons) * persons;
     }
 
-    total = round(bill + tip);
+    // round total
+    if (totalRounding && doRoundToQuarter){
+        total = roundToQuarter((bill + tip) / persons) * persons;
+    }
+    else if (totalRounding){
+        total = roundWeighted((bill + tip) / persons) * persons;
+    }
+    else {
+        total = round((bill + tip) / persons) * persons;
+    }
+
+    // adjust tip if total has changed
+    if (totalRounding){
+        tip = round(total - bill);
+    }
+
+    if (totalRounding || tipRounding){
+        float newTipPercent = round((tip/adjustedBill)*100,1);
+        ui->newTipLabel->show();
+        ui->newTipDisplay->show();
+        ui->newTipDisplay->setText(QString::number(newTipPercent));
+    }
+    else {
+        ui->newTipLabel->hide();
+        ui->newTipDisplay->hide();
+    }
+
+    // last minute rounding just in case
     personTotal = round(total / persons);
     personTip = round(tip / persons);
 
-    tip = round(tip);
-
-    ui->totalTipResultLabel->setText(QString::number(tip));
-    ui->totalResultLabel->setText(QString::number(total));
-    ui->personTotalResultLabel->setText(QString::number(personTotal));
-    ui->personTipResultLabel->setText(QString::number(personTip));
-}
-
-void TipQalc::roundTotal()
-{
-
-}
-
-void TipQalc::roundTip()
-{
-
+    ui->totalTipResultLabel->setText(floatToDisplay(tip));
+    ui->totalResultLabel->setText(floatToDisplay(total));
+    ui->personTotalResultLabel->setText(floatToDisplay(personTotal));
+    ui->personTipResultLabel->setText(floatToDisplay(personTip));
 }
 
 /*
@@ -272,17 +343,14 @@ void TipQalc::numberClicked(QString value)
     QRegExp rx("");
 
     // set decimal precision according to field
-    if (objectName == "billLine")
-    {
+    if (objectName == "billLine"){
         rx.setPattern("\\.\\d\\d");
     }
-    else if (objectName == "taxLine")
-    {
+    else if (objectName == "taxLine"){
         rx.setPattern("\\.\\d\\d\\d");
     }
 
-    if (!editableLine->text().contains(rx))
-    {
+    if (!editableLine->text().contains(rx)){
         adjustCursorPosition();
         editableLine->insert(value);
     }
@@ -290,8 +358,7 @@ void TipQalc::numberClicked(QString value)
     // remove leading 0
     rx.setPattern("^0\\d");
     QString lineValue = editableLine->text();
-    if (lineValue.contains(rx))
-    {
+    if (lineValue.contains(rx)) {
         lineValue.remove(0,1);
     }
     editableLine->clear();
@@ -303,8 +370,7 @@ void TipQalc::numberClicked(QString value)
 */
 void TipQalc::periodClicked()
 {
-    if (!editableLine->text().contains("."))
-    {
+    if (!editableLine->text().contains(".")) {
         adjustCursorPosition();
         editableLine->insert(".");
     }
@@ -334,3 +400,15 @@ void TipQalc::on_calculateButton_clicked()
 {
     calculateTip();
 }
+
+void TipQalc::on_roundTotalButton_clicked()
+{
+    ui->roundTipButton->setChecked(false);
+    calculateTip();
+}
+
+void TipQalc::on_roundTipButton_clicked()
+{
+    ui->roundTotalButton->setChecked(false);
+    calculateTip();
+}
